Class: Lutaml::Xsd::Commands::CoverageCommand

Inherits:
BaseCommand
  • Object
show all
Defined in:
lib/lutaml/xsd/commands/coverage_command.rb

Overview

Command for analyzing schema coverage from entry points Shows which types are reachable from specified entry types

Instance Attribute Summary

Attributes inherited from BaseCommand

#options

Instance Method Summary collapse

Constructor Details

#initialize(package_path, options) ⇒ CoverageCommand

Returns a new instance of CoverageCommand.



11
12
13
14
15
16
# File 'lib/lutaml/xsd/commands/coverage_command.rb', line 11

def initialize(package_path, options)
  super(options)
  @package_path = package_path
  @entry_types = parse_entry_types(options[:entry])
  @format = options[:format] || "text"
end

Instance Method Details

#runObject



18
19
20
21
# File 'lib/lutaml/xsd/commands/coverage_command.rb', line 18

def run
  validate_package_exists
  perform_coverage_analysis
end